;********************************************************************
;                                                                   *
; PPPPP  HH   H  OOOO  EEEEEE NN   N IIIIII X   XX                  *
; PP   P HH   H OO   O EE     NNN  N   II    X XX                   *
; PP   P HHHHHH OO   O EEEEE  NN N N   II     XX                    *
; PPPPP  HH   H OO   O EE     NN N N   II    XXX                    *
; PP     HH   H OO   O EE     NN  NN   II   XXX X                   *
; PP     HH   H  OOOO  EEEEEE NN   N IIIIII XX   X - RECOVERED      *
;                                                                   *
; by SORBAS  2020/02/01                                             *
;********************************************************************
; Arcade space shooter for vertical monitor orientation,
; written for the Commodore C16/Plus4 with 64k,
; using the text mode with rotated user defined characters.
; This is the first known Commodore game using a vertical orientation.
; The basic game idea and design is mostly taken from the
; Amstar Electronics 'Phoenix' arcade game (1980).
;********************************************************************
; Notes: this source is formatted for the ACME 0.96.4 cross assembler.
; Compression is handled with Exomizer 2.0.11.
; build.cmd will call both to create an assembled file and then the
; crunched release version.
;********************************************************************
; Usage:
; Joystick at JOY2.
; Joystick UP or DOWN activates the force shield.
;********************************************************************

;Work in progress:
; * the bird level, BirdsUpdate

;********************************************************************
; SYMBOLIC CONSTANTS                                                *
;********************************************************************
FALSE               = $00
TRUE                = $01

HIGH_NIBBLE_MASK    = $F0
LOW_NIBBLE_MASK     = $0F

;Color and lumi values **********************************************
;BLACK               = %....0000
;WHITE               = %....0001
;RED                 = %....0010
;CYAN                = %....0011
;PURPLE              = %....0100
;GREEN               = %....0101
;BLUE                = %....0110
;YELLOW              = %....0111
;ORANGE              = %....1000
;BROWN               = %....1001
;YLGRN               = %....1010
;PINK                = %....1011
;BLGRN               = %....1100
;LBLU                = %....1101
;DBLU                = %....1110
;LGRN                = %....1111
;
;LUMI_0              = %0000....
;LUMI_1              = %0001....
;LUMI_2              = %0010....
;LUMI_3              = %0011....
;LUMI_4              = %0100....
;LUMI_5              = %0101....
;LUMI_6              = %0110....
;LUMI_7              = %0111....

BLK                 = $00           ;color:black        lumi:0
WHT                 = $71           ;color:white        lumi:7
RED5                = $52           ;color:red,         lumi:5
RED3                = $32           ;color:red,         lumi:3
CYN6                = $63           ;color:cyan,        lumi:6
CYN7                = $73
CYNF                = $E3           ;color:cyan,        lumi:6, flash
PUR                 = $44           ;color:purple,      lumi:4
PUR7                = $74
GRN                 = $45           ;color:green,       lumi:4
BLU                 = $26           ;color:blue,        lumi:2
YEL6                = $67           ;color:yellow,      lumi:6
YEL7                = $77
ORG                 = $68           ;color:orange,      lumi:6
ORG5                = $58           ;color:orange,      lumi:5
YGRN                = $5A           ;color:yellow-green lumi:5
YGR6                = $6A           ;color:yellow-green lumi:6
PNK2                = $2B           ;color:pink,        lumi:2
PNK3                = $3B           ;color:pink,        lumi:3
PNK5                = $5B           ;color:pink,        lumi:5
PNK                 = $6B           ;color:pink,        lumi:6
BGR4                = $4C           ;color:blue-green,  lumi:4
BGRN                = $5C           ;color:blue-green,  lumi:5
LBLU                = $6D           ;color:light-blue,  lumi:6
LGRN                = $65           ;color:light-green, lumi:6

SCCO                = $54           ;color of all decimal charecters
SCCF                = $D4           ;color of the flashing ones

;unique colors for collission detection
SHIP_COLOR_1        = BGR4
SHIP_COLOR_2        = BGRN
SHIP_COLOR_3        = PNK3          ;during active shield
SHIP_COLOR_4        = PNK5          ;during active shield

;Character codes masks and setup ************************************
CHR_ASTERISK        = $1E           ;'*'
CHR_ZERO            = $20           ;'0'
CHR_SPACE           = $2A           ;' '
CHR_PLAYERBULLET    = $2C           ;player bullet
CHR_ALIENBULLET1    = $2D           ;alien bullet 1
CHR_STAR1           = $30           ;1st of the 8 star char's.
CHR_SHIELD_START    = $44           ;1st of the player shield char's
CHR_SHIELD_END      = $50           ;last player shield char + 1

CHRGROUP_PROTBAR    = $50
CHRGROUP_DAMAGE     = $70
CHRGROUP_BOSS       = $F0

STARS_DEL_MASK      = %11111000     ;to check if position is empty or star

CHAR_SETA_BASE      = %01000000     ;16*4=64 => $4000 for CharacterSetA
CHAR_SETB_BASE      = %01010000     ;16*5=80 => $5000 for CharacterSetB
CHAR_SETC_BASE      = %01100000     ;16*6=96 => $6000 for CharacterSetC

READ_FROM_RAM       = %11111011     ;read char set from RAM
RVS_VIDEO_SW        = %10000000     ;256 user defined characters

MEM_CONFIG_RAM_IO_RAM           = %00110101     ;motor:off, check-rec:on, RAM-I/O-RAM

COLOR_SET1          = $00
COLOR_SET2          = $01

;Raster codes *******************************************************
V_SCAN_LINE         = $080          ;vertical scan pos line for interrupt

;Game stars constants ***********************************************
STARS_NUM_ROWS      = $19
STARS_NUM_FRAMES    = $08

;Game flow  constants ***********************************************
FLOW_NUM_LIVES      = $03
LENGHT_OF_OVERALLGAMEFLOWTABLE  = $0A
FLOW_WAVE_1         = $01           ;for the waves with aliens (form A - D)
FLOW_WAVE_2         = $02           ;for the waves with birds (form A - B)
FLOW_WAVE_3         = $03           ;for the wave with mothership (boss incl aliens on top)

FLOW_ALIEN_FORM_A       = $00
FLOW_ALIEN_FORM_B       = $01
FLOW_ALIEN_FORM_C       = $02
FLOW_ALIEN_FORM_D       = $03
FLOW_ALIEN_FORM_E       = $04
FLOW_ALIEN_FORM_F       = $05

FLOW_BIRD_FORM_A        = $00
FLOW_BIRD_FORM_B        = $01

;states
FLOW_STATE_TITLE        = $00       ;for the general title execution flow
FLOW_STATE_SLOWPRINT    = $01       ;for all slow printing lines
FLOW_STATE_ANIMATION    = $02       ;for the bird animation on 2nd title page
FLOW_STATE_PUSHBUTTON   = $03       ;for one or two player page
FLOW_STATE_INIT         = $04
FLOW_STATE_MENU         = $05
FLOW_STATE_CURTAINUP    = $06       ;for mothership landing
FLOW_STATE_FADEIN       = $07
FLOW_STATE_ALIVE        = $08
FLOW_STATE_DYING        = $09
FLOW_STATE_GAMEOVER     = $0A
FLOW_STATE_THEEND       = $0B       ;-)

;commands
CMD_WAIT                = $01       ;just wait 
CMD_PRINT_TOP_LINES     = $02       ;print the top lines
CMD_PRINT_BOTTOM_LINES  = $03       ;print the bottom lines
CMD_PRINT_START_TEXT    = $04
CMD_TO_STATE_SLOWPRINT  = $05       ;change state to:FLOW_STATE_SLOWPRINT
CMD_DRAW_4_SHAPES       = $06       ;draw the 4 shapes at score average table
CMD_CLEAR_SCREEN        = $07       ;clear the screen
CMD_START_STARFIELD     = $08       ;start the background starfield animation
CMD_STOP_STARFIELD      = $09       ;stop the background starfield animation
CMD_TO_STATE_ANIMATION  = $0A       ;change state to:FLOW_STATE_ANIMATION
CMD_RESET_POINTERS      = $0B       ;reset pointer for game title execution table,
                                    ;game title shape table, game title character table

;screen positions
SCREEN_WIDTH        = $28           ;for screen boundary
SCREEN_HIGHT        = $19

SCORE_NUM_X_POS     = $26           ;for decimal values
SCORE_NUM_Y_POS     = $00
LIVES_NUM_X_POS     = $25
LIVES_NUM_Y_POS     = $03
HI_NUM_X_POS        = $26
HI_NUM_Y_POS        = $09
COIN_NUM_X_POS      = $25
COIN_NUM_Y_POS      = $0D

SCORE2_NUM_X_POS    = $26
SCORE2_NUM_Y_POS    = $13
LIVES2_NUM_X_POS    = $25
LIVES2_NUM_Y_POS    = $16

START_TEXT_X_POS    = $1B           ;'PUSH 1PLAYER ..' position
START_TEXT_Y_POS    = $0A

GAMEOVER_TEXT_X_POS = $1B           ;'GAME OVER' position
GAMEOVER_TEXT_Y_POS = $07

;score point values
TWENTY_POINTS       = $20           ;used for: IncreaseScoreLow
FOURTY_POINTS       = $40           ;
FIFTY_POINTS        = $50           ;
EIGHTY_POINTS       = $80           ;
ONEHUNDRED_POINTS   = $01           ;used for: IncreaseScoreHigh
TWOHUNDRED_POINTS   = $02           ;
FOURHUNDRED_POINTS  = $04           ;
EIGHTHUNDRED_POINTS = $08           ;
ONETHOUSAND_POINTS  = $10           ;
NINETHOUSAND_POINTS = $90           ;

;bonus threshold value
BONUS_ONE_LIVE      = $60           ;if score grater 6000

FRAME_MASK          = %00000100     ;every 5th frame

;Input constants ****************************************************
JOYSTICK_PORT_2     = %11111101     ;$FD: port2

UP_MASK             = %00000001
DOWN_MASK           = %00000010
LEFT_MASK           = %00000100
RIGHT_MASK          = %00001000
FIRE_MASK           = %10000000
FIRE_DELAY_MAX      = $8E

;Player ship constants **********************************************
PLAYER_START_X      = $04           ;only vertical movement
PLAYER_START_Y      = $0B           ;1-24
PLAYER_X_MIN        = $04           ;only vertical movement
PLAYER_X_MAX        = $04           ;only vertical movement
PLAYER_Y_MIN        = $00
PLAYER_Y_MAX        = $17

;Explosion constants ************************************************
EXPLOSION_TYPE_A    = $01
EXPLOSION_TYPE_B    = $02
NO_OF_PARTICLE_EXPOSION_FRAMES    = $2B

;Alien constants ****************************************************
ALIENS_MAX          = $10

RIGHT               = $00
LEFT                = $01
PRESENT             = $01
DELETED             = $00

;mode of alien
CONVOY_MODE         = $00
ATTACK_MODE         = $01
ESCAPE_MODE         = $02
GRAPPER_MODE        = $03

NR_OF_PICKUP_TASKS  = $04

;commands to trigger actions during alien movements
CMD_ATTACK          = $10           ;decouple and move down from convoy
CMD_ATTACK_SHOOT    = $11           ;move down and shoot
CMD_GRAPPER         = $30           ;move left/right, be a grapper (no shoots)
CMD_CONVOY          = $40           ;move left/right, as part of the convoy

ALIEN_SHAPE_00      = $00           ;for delete

ALIEN_SHAPE_01      = $01
ALIEN_SHAPE_02      = $02
ALIEN_SHAPE_03      = $03
ALIEN_SHAPE_04      = $04
ALIEN_SHAPE_05      = $05
ALIEN_SHAPE_06      = $06
ALIEN_SHAPE_07      = $07
ALIEN_SHAPE_08      = $08
ALIEN_SHAPE_09      = $09
ALIEN_SHAPE_10      = $0A
ALIEN_SHAPE_11      = $0B
ALIEN_SHAPE_12      = $0C
ALIEN_SHAPE_13      = $0D
ALIEN_SHAPE_14      = $0E
ALIEN_SHAPE_15      = $0F
ALIEN_SHAPE_16      = $10
ALIEN_SHAPE_17      = $11
ALIEN_SHAPE_18      = $12
ALIEN_SHAPE_19      = $13
ALIEN_SHAPE_20      = $14
ALIEN_SHAPE_21      = $15
ALIEN_SHAPE_22      = $16
ALIEN_SHAPE_23      = $17
ALIEN_SHAPE_24      = $18
ALIEN_SHAPE_25      = $19
ALIEN_SHAPE_26      = $1A
ALIEN_SHAPE_27      = $1B
ALIEN_SHAPE_28      = $1C
ALIEN_SHAPE_29      = $1D
ALIEN_SHAPE_30      = $1E
ALIEN_SHAPE_31      = $1F
ALIEN_SHAPE_32      = $20
ALIEN_SHAPE_33      = $21
ALIEN_SHAPE_34      = $22
ALIEN_SHAPE_35      = $23
ALIEN_SHAPE_36      = $24
ALIEN_SHAPE_37      = $25
ALIEN_SHAPE_38      = $26
ALIEN_SHAPE_39      = $27

ATTACK_TABLES_MAX   = $05
ANGRY_TABLES_MAX    = $02

;alien behavior states **********************************************
ALIEN_STATE_PICKUP  = $00           ;pickup and attack/escape/shoot movement
ALIEN_STATE_GRAPPER = $01           ;let all grappers escape/shoot back to convoy
ALIEN_STATE_ANGRY   = $02           ;do the angry convoy move further down

;Mothership constants ***********************************************
NO_OF_BUNKER_EDGES  = $14
SIZE_OF_PURPLE_LAYER    = $12
BUNKER_FRAME_INDEX_10   = $10       ;for bunkerDamageFrame17
BOSS_FIRE_DELAY     = $3A

;boss behavior states ***********************************************
BOSS_STATE_ACTIVE   = $00
BOSS_STATE_HIT      = $01
BOSS_STATE_EXPL     = $02
BOSS_STATE_SCORE    = $03

;Bird constants *****************************************************
BIRDS_MAX           = $08

BIRD_SHAPE_00       = $00           ;for delete

BIRD_SHAPE_01       = $01
BIRD_SHAPE_02       = $02
BIRD_SHAPE_03       = $03
BIRD_SHAPE_04       = $04
BIRD_SHAPE_05       = $05
BIRD_SHAPE_06       = $06
BIRD_SHAPE_07       = $07
BIRD_SHAPE_08       = $08
BIRD_SHAPE_09       = $09
BIRD_SHAPE_10       = $0A
BIRD_SHAPE_11       = $0B
BIRD_SHAPE_12       = $0C
BIRD_SHAPE_13       = $0D
BIRD_SHAPE_14       = $0E
BIRD_SHAPE_15       = $0F
BIRD_SHAPE_16       = $10
BIRD_SHAPE_17       = $11
BIRD_SHAPE_18       = $12
BIRD_SHAPE_19       = $13
BIRD_SHAPE_20       = $14
BIRD_SHAPE_21       = $15
BIRD_SHAPE_22       = $16
BIRD_SHAPE_23       = $17
BIRD_SHAPE_24       = $18
BIRD_SHAPE_25       = $19
BIRD_SHAPE_26       = $1A
BIRD_SHAPE_27       = $1B
BIRD_SHAPE_28       = $1C
BIRD_SHAPE_29       = $1D
BIRD_SHAPE_30       = $1E

EO_BIRDYANIMATIONTABLE  = $AA
EO_BIRDXANIMATIONTABLE  = $AA

;Bullets constants **************************************************
BULLETS_MAX         = $10           ;max size of the bullet queue

;Timer constants ****************************************************
TIMERS_MAX          = $11           ;individual software timers
TIMER_0             = $00           ;title animation
TIMER_1             = $01           ;ship movement
TIMER_2             = $02           ;ship animation
TIMER_3             = $03           ;shield animation
TIMER_4             = $04           ;shield duration
TIMER_5             = $05           ;shield downtime
TIMER_6             = $06           ;alien animation
TIMER_7             = $07           ;play break
TIMER_8             = $08           ;alien explosion animation
TIMER_9             = $09           ;game speed (until next speed up)
TIMER_A             = $0A           ;player explosion animation
TIMER_B             = $0B           ;alien fade in animation
TIMER_C             = $0C           ;alien attack interval time
TIMER_D             = $0D           ;alien attack movement speed
TIMER_E             = $0E           ;mothership movement speed
TIMER_F             = $0F           ;alien pilot animation speed
TIMER_G             = $10           ;birds x movement speed

;timer states
TIMER_STATE_INACTIVE    = $00       ;timer is not in use
TIMER_STATE_ACTIVE      = $01       ;timer is running
TIMER_STATE_THRESHOLD   = $02       ;timer threshold reached

;timer threshold values
;TIMER_0
TITLE_TIMER_VALUE_H = $00           ;title animation speed
TITLE_TIMER_VALUE_L = $05
;TIMER_1
PLAYER_SPEED_TIMER_VALUE_H = $00    ;ship movement speed
PLAYER_SPEED_TIMER_VALUE_L = $05
;TIMER_2
PLAYER_FRAME_TIMER_VALUE_H = $00    ;ship animation speed
PLAYER_FRAME_TIMER_VALUE_L = $0A
;TIMER_3
SHIELD_FRAME_TIMER_VALUE_H = $00    ;shield animation speed
SHIELD_FRAME_TIMER_VALUE_L = $04
;TIMER_4
SHIELD_DURATION_TIMER_VALUE_H = $00 ;shield duration time
SHIELD_DURATION_TIMER_VALUE_L = $60
;TIMER_5
SHIELD_FORBIDDEN_TIMER_VALUE_H = $00 ;shield reuse forbidden time
SHIELD_FORBIDDEN_TIMER_VALUE_L = $D0
;TIMER_6
ALIEN_SPEED_TIMER_VALUE_H = $00 ;alien movement/animation speed
ALIEN_SPEED_TIMER_VALUE_L = $06
;TIMER_7
PLAY_BREAK_TIMER_VALUE_H = $00 ;play break time
PLAY_BREAK_TIMER_VALUE_L = $78
;TIMER_8
ALIEN_EXPLOSION_FRAME_TIMER_VALUE_H = $00 ;alien explosion animation speed
ALIEN_EXPLOSION_FRAME_TIMER_VALUE_L = $04
;TIMER_9
GAME_SPEED_TIMER_VALUE_H = $04 ;time until next speed up
GAME_SPEED_TIMER_VALUE_L = $50
;TIMER_A
PLAYER_EXPLOSION_FRAME_TIMER_VALUE_H = $00 ;player explosion animation speed
PLAYER_EXPLOSION_FRAME_TIMER_VALUE_L = $03
;TIMER_B
ALIEN_FADEIN_FRAME_TIMER_VALUE_H = $00 ;alien fade in animation speed
ALIEN_FADEIN_FRAME_TIMER_VALUE_L = $06
;TIMER_C
ALIEN_ATTACK_INTERVAL_TIMER_VALUE_H = $00    ;time until next attack
ALIEN_ATTACK_INTERVAL_TIMER_VALUE_L = $50
;TIMER_D
ALIEN_ATTACK_SPEED_TIMER_VALUE_H = $00    ;alien attack movement speed
ALIEN_ATTACK_SPEED_TIMER_VALUE_L = $04
;TIMER_E
BOSS_MOVEMENT_SPEED_TIMER_VALUE_H = $00    ;mothership movement speed
BOSS_MOVEMENT_SPEED_TIMER_VALUE_L = $7E
;TIMER_F
ALIENPILOT_FRAME_TIMER_VALUE_H = $00    ;alien pilot animation speed
ALIENPILOT_FRAME_TIMER_VALUE_L = $01
;TIMER_G
BIRDSX_SPEED_TIMER_VALUE_H = $00    ;birds x movement speed
BIRDSX_SPEED_TIMER_VALUE_L = $05

;Sound constants ****************************************************
VOICE_1             = $00           ;Oscillator1 Squarewave only
VOICE_2             = $01           ;Oscillator2 Squarewave only
VOICE_3             = $02           ;Noise (shared with voice 2)
VOICES_MAX          = $03

SQUARE1_START       = $10
SQUARE1_END         = $11
SQUARE2_START       = $20
SQUARE2_END         = $21
NOISE_START         = $30
NOISE_END           = $31

CMD_VOICE           = $C0
CMD_VOLUME          = $C1
CMD_FREQUENCYHIGH   = $C2
CMD_FREQUENCYLOW    = $C3
CMD_DELAY           = $C4
CMD_END             = $C5

;Sound masks ********************************************************
VOLUME_MASK         = %11110000
FREQHIGH_MASK       = %11111100
VOICE1_MASK         = %11101111
VOICE2_MASK         = %11011111
NOISE_MASK          = %10111111
MUTE_MASK           = %01111111

VOICE1_ON           = %00010000
VOICE2_ON           = %00100000
NOISE_ON            = %01000000
MUTE_ON             = %10000000     ;all sounds off

;********************************************************************
; COMMODORE C16 SYSTEM ADRESSES                                     *
;********************************************************************
;Zero page
D6510                   = $00       ;I/O Data direction register
R6510                   = $01       ;I/O Port register

FLEET                   = $02
FLEETINDEX              = $03
FLEETX                  = $04
FLEETY                  = $05
FLEETOLDX               = $06
FLEETOLDY               = $07
ALIENSTEMP              = $08

PLAYERACTIVE            = $09
PLAYERXCHAR             = $0A
PLAYERXOFFSET           = $0B
PLAYERYCHAR             = $0C
PLAYERYOFFSET           = $0D
PLAYERYOLD              = $0E
PLAYERINVULNERABLE      = $0F       ;TRUE during active shield

BULLETSXCHARCURRENT     = $10
BULLETSYCHARCURRENT     = $11
BULLETSYOFFSETCURRENT   = $12
BULLETSCOLORCURRENT     = $13
BULLETSDIRCURRENT       = $14
BULLETSTEMP             = $15
BULLETSYFLAG            = $16
BULLETSXCHARCOL         = $17
BULLETSYCHARCOL         = $18
BULLETSDIRCOL           = $19
ALIENBULLETSYTEMP       = $1A       ;used to calculate the right part of alien character
ALIENBULLETTOGGLE       = $1B

FADEINALIENFINISHED     = $1C
                        ;$1D
                        ;$1E
BIRDXMOVEVALUE          = $1F

BIRDYMOVEOFFSETL        = $20
BIRDYMOVEOFFSETH        = $21
FADEINBIRDSFRAMEINDEX   = $22
BIRDFRAMEINDEX          = $23
BIRDXMOVEL              = $24
BIRDXMOVEH              = $25
BIRDSTEMP               = $26
BIRDFLEET               = $27
BIRDFLEETINDEX          = $28
BIRDFLEETX              = $29
BIRDFLEETY              = $2A
BIRDFLEETOLDX           = $2B
BIRDFLEETOLDY           = $2C
BIRDSHAPETYPE           = $2D
FADEINBIRDFINISHED      = $2E
BIRDXMOVETIMERVALUE     = $2F


ALIENPILOTFRAMEL        = $30       ;Pointer to frame data
ALIENPILOTFRAMEH        = $31
ALIENPILOTFRAMEINDEX    = $32
CURRENTBOSSXPOS         = $33

CHRXPOS                 = $34
CHRYPOS                 = $35
SHAPEDATAINDEX          = $36
MAXSCREENX              = $37

BUNKERDAMAGEFRAMEL      = $38       ;Pointer to bunker damage frame data
BUNKERDAMAGEFRAMEH      = $39
BUNKERADDRESSL          = $3A       ;Bunker address
BUNKERADDRESSH          = $3B
HITCHARACTER            = $3C
MOTHERSHIPSCORE         = $3D
BOSSBEHAVIORL           = $3E       ;states: active, hit, exposion, score
BOSSBEHAVIORH           = $3F

SOUNDCMDL               = $40       ;Pointer: sound voice command
SOUNDCMDH               = $41       ;
VOLUME                  = $42
FREQ1LOW                = $43
FREQ1HIGH               = $44
FREQ2LOW                = $45
FREQ2HIGH               = $46

PROXIMITY               = $47       ;distance to the enemy
GAMESPEEDINDEX          = $49
ANGRYTABLEINDEX         = $4A

CURRENTWAVE             = $4B       ;the 3 different kind of games
CURRENTALIENFORM        = $4C       ;alien formations: A, B, C, D
CURRENTBIRDFORM         = $4D       ;bird formations: A, B
CURRENTBOSSFORM         = $4E       ;boss formations: A, B
OVERALLGAMEINDEX        = $4F       ;index of overAllGameFlowTable,
                                    ;representing the levels of the game.

SCREENRAML              = $50       ;Pointer: screen RAM for DrawShape
SCREENRAMH              = $51       ;
COLORRAML               = $52       ;Pointer: color RAM for DrawShape
COLORRAMH               = $53       ;

ATTACKPATTERNL          = $54       ;Pointer: alien attack pattern table
ATTACKPATTERNH          = $55       ;

BLOCKWIDTH              = $56       ;Current width of a shape
BLOCKHEIGHT             = $57       ;Current height of a shape

CONVOYFRAMEINDEX        = $58       ;Current frame number for the convoy aliens
ALIENEXPLOSIONINDEX     = $59       ;Current frame number for the alien explosion
PARTICLEEXPLOSIONINDEX  = $5A       ;Current frame number for the particle explosion
PLAYERFRAME             = $5B       ;Current frame number for the player animation
SHIELDFRAME             = $5C       ;Current frame number for the shield
FADEINALIENFRAMEINDEX   = $5D       ;Current frame number for the fade in animation

TITLECHRLINEINDEX       = $5E       ;Current index for the slow printed game intro lines
TITLECOLUMNINDEX        = $5F       ;Current index for the slow printed game intro colums

ATTACKPOINTER           = $60       ;Pointer: alien#1 attack pattern table
                                    ;$60 - 7F
                                    ;Pointer: alien#16 attack pattern table

CURRENTALIENNUMBER      = $80       ;Current alien number (0..15)
CURRENTSHAPEXPOS        = $81       ;Current shape x position (0..39)
CURRENTSHAPEYPOS        = $82       ;Current shape y position (0..24)
CURRENTSHAPEXOFFSET     = $83       ;Current shape x offset
CURRENTSHAPEYOFFSET     = $84       ;Current shape y offset
CURRENTALIENCOMMAND     = $85       ;Current alien command (CMD_ATTACK, CMD_ATTACK_SHOOT, CMD_ESCAPE...)
CURRENTSHAPENUMBER      = $86       ;Current shape number

CURRENTXPOS             = $87       ;Current x position for DrawTextRotated, DrawDecimalRotated
CURRENTYPOS             = $88       ;Current y position for DrawTextRotated, DrawDecimalRotated
CURRENTCHARACTER        = $89       ;Current screen character code for DrawTextRotated
CURRENTCOLOR            = $8A       ;Current screen color code for DrawTextRotated, Stars

STARSALLOWED            = $8B       ;start/stop background animation
STARSCURRENTX           = $8C       ;Current x position of star
STARSCURRENTY           = $8D       ;Current y position of star
STARSCURRENTFRAME       = $8E       ;Current character code from starsFrameArray
STARSCURRENTCOLOR       = $8F       ;Current color code of star
STARSNEXTSPEED          = $90       ;Current speed of a star

PLAYER1SCORELOW         = $91       ;Player one, low score part (ones and tens)
PLAYER1SCOREMID         = $92       ;Player one, mid score part (hundreds and thousands)
PLAYER1SCOREHIGH        = $93       ;Player one, high score part (ten-thousands and hundred-thousands)
PLAYER1LIVES            = $94       ;Player one, number of lives

PLAYER2SCORELOW         = $95       ;Player two, low score part (ones and tens)
PLAYER2SCOREMID         = $96       ;Player two, mid score part (hundreds and thousands)
PLAYER2SCOREHIGH        = $97       ;Player two, high score part (ten-thousands and hundred-thousands)
PLAYER2LIVES            = $98       ;Player two, number of lives

HIGHSCORELOW            = $99       ;High score low part
HIGHSCOREMID            = $9A       ;High score mid part
HIGHSCOREHIGH           = $9B       ;High score high part
COIN                    = $9C       ;Number of coins

CONVOYPOSITION          = $9D       ;initial 03
CONVOYLEFTMAX           = $9E       ;00 or 02
CONVOYRIGHTMAX          = $9F       ;05 or 07

MULTIPLIER              = $A0       ;8-bit unsigned multiplier for UnsMultiplication
MULTIPLICAND            = $A1       ;8-bit unsigned multiplicand for UnsMultiplication
PRODUCTL                = $A2       ;16-bit unsigned product lsb for UnsMultiplication
PRODUCTH                = $A3       ;16-bit unsigned product msb for UnsMultiplication

CURRENTDECNUMBER        = $A4       ;Current 2 nybbles of: scores,lives,hi

SPIRALFILLCHAR          = $A5       ;Holding the asterisk character
SPIRALFILLCOLOR         = $A6       ;Holding the color of the character
SPIRALFILLCOLORARRAYINDEX   = $A7   ;Index for the color array
SPIRALFILLSCREENADDRL   = $A8
SPIRALFILLSCREENADDRH   = $A9
SPIRALFILLCOLORADDRL    = $AA
SPIRALFILLCOLORADDRH    = $AB
SPIRALFILLWIDTH         = $AC
SPIRALFILLHEIGHT        = $AD

VICTORYSTATE            = $AE
FLOWTITLESTARTED        = $AF

TITLEEXECPOINTER        = $B0       ;Pointer: game title execution table
                                    ;
TITLEBIRDFRAMEINDEX     = $B2       ;Index for bird frames at game intro
                                    
TITLECHRPOINTER         = $B4       ;Pointer: game title character table
                                    ;
TITLECOLORPOINTER       = $B6       ;Pointer: game title color code table
                                    ;

TITLECMD                = $B8       ;Current command value for the intro execution
TITLETIMER              = $B9       ;Next timer value for the intro execution 

MEMORYRANGE             = $BA       ;Amount of bytes to be copied
NEWRANDOMSEED           = $BB       ;A new value for the random seed
WAITIRQ                 = $BC       ;1 if irq rountines are finished

CHRSETBASE              = $BD       ;High byte of base address of current character set

DIFFICULTY              = $BE
FLOWSTATE               = $BF       ;default is 00: FLOW_STATE_TITLE

ALIENBEHAVIORL          = $C0       ;Pointer: Jump table for the aliens behavior
ALIENBEHAVIORH          = $C1       ;

TITLECOLORSET           = $C2       ;
ALIENCOLORSET           = $C3       ;

PLAYERALIENCOLLISION    = $C4       ;true if alien has hit the player ship
REPEATCOUNTER           = $C5

CURRENTBUNKERSEDGEINDEX = $C6
                        ;$C7

EXPLOSIONBPOINTERL      = $C8
EXPLOSIONBPOINTERH      = $C9
PARTICLEEXPLOSIONPOINTERL = $CA
PARTICLEEXPLOSIONPOINTERH = $CB
SHIELDXPOS              = $CC
SHIELDYPOS              = $CD
EXPLOSIONSPENDING       = $CE

BONUSP1FLAG             = $CF       ;Bonus flag for player 1

SCREENL                 = $D0       ;Pointer: screen/color RAM for DrawTextRotated
SCREENH                 = $D1       ;
GAMEFLOWL               = $D2       ;Pointer: game flow
GAMEFLOWH               = $D3       ;
STRINGL                 = $D4       ;Pointer: strings
STRINGH                 = $D5       ;
TEXTCOLORL              = $D6       ;Pointer: text color
TEXTCOLORH              = $D7       ;
SPIRALSCREENL           = $D8       ;Pointer: spiral fill top left corner screen ram
SPIRALSCREENH           = $D9       ;
SPIRALCOLORL            = $DA       ;Pointer: spiral fill top left corner color ram
SPIRALCOLORH            = $DB       ;
SPIRALWIDTH             = $DC       ;Spiral fill current width
SPIRALHEIGHT            = $DD       ;Spiral fill current height 
STARSCREENL             = $DE       ;Pointer: screen/color RAM for stars
STARSCREENH             = $DF       ;
BULLETSCREENL           = $E0       ;Pointer: screen/color RAM for bullets
BULLETSCREENH           = $E1       ;
SOURCEADDRESS           = $E2       ;Pointer: source address for copy memory
TARGETADDRESS           = $E4       ;Pointer: target address for copy memory
PSEUDORANDOML           = $E6       ;16-bit pseudo random number
PSEUDORANDOMH           = $E7       ;
FRAMECOUNTER            = $E8       ;genaral purpose 8 bit frame counter
VALUETOSCORE            = $E9       ;value to add to the score

GAMEPORTTHISFRAME       = $EA       ;
GAMEPORTLASTFRAME       = $EB       ;
GAMEPORTDIFF            = $EC       ;
UPKEY                   = $ED       ;
FIREBLIP                = $EE       ;
FIREDELAY               = $EF       ;

SEEDVALUEUPDATED        = $F0       ;

ZP_LAST                 = SEEDVALUEUPDATED

;Video RAM
COLORRAM                = $0800     ;Color ram start address
SCREENRAM               = $0C00     ;Screen ram start address

;TED chip registers ($FF00-$FF3F)
TEDREG07                = $FF07     ;Bit 7:RVS-Video (0=HW, 1=SW)
TEDREG08                = $FF08     ;Keyboard matrix
TEDREG09                = $FF09     ;Interrupt sources
TEDREG0A                = $FF0A     ;Interrupt masking
TEDREG0B                = $FF0B     ;Raster comparison (Bit 0-7)
TEDREG0E                = $FF0E     ;Low byte of voice 1 frequency
TEDREG0F                = $FF0F     ;Low byte of voice 2 frequency
TEDREG10                = $FF10     ;Top 2 bits of voice 2 frequency
TEDREG11                = $FF11     ;Volume, Voice select, sound reload bit
TEDREG12                = $FF12     ;Bit0,1: voice 1 freq high, Bit 2:RAM/ROM Bank (0=RAM, 1=ROM)
TEDREG13                = $FF13     ;CB5,CB4,CB3,CB2,CB1,CB0,SClock,ST
TEDREG15                = $FF15     ;Background 0 Color/Lumi
TEDREG16                = $FF16     ;Background 1 Color/Lumi
TEDREG17                = $FF17     ;Background 2 Color/Lumi
TEDREG18                = $FF18     ;Background 3 Color/Lumi
TEDREG19                = $FF19     ;Frame Color/Lumi
TEDREG1E                = $FF1E     ;Current raster column (Bit 1-8)
TEDREG3F                = $FF3F     ;RAM select (write only)

;Kernal jump table ($FF49-$FFFF)
FF3EL                   = $FFF7     ;low byte address of :'STA $FF3E ;switch-on ROM'
KERNALRESETL            = $FFFA     ;low byte address of :'JMP $F2A4 ;jump to reset routine'
KERNALRESETH            = $FFFB     ;high byte address of :'JMP $F2A4 ;jump to reset routine'
CPUINTL                 = $FFFE     ;Processor Interrupt
CPUINTH                 = $FFFF

;********************************************************************
; BASIC CODE LINE                                                   *
;********************************************************************
; 10 SYS (4112)
* = $1001
                !byte $0e,$10,$0a,$00,$9e,$20,$28,$34
                !byte $31,$31,$32,$29,$00,$00,$00

;********************************************************************
; MAIN ENTRY POINT                                                  *
;********************************************************************
* = $1010
.main_init
                ;stop interrupts
                sei
                ;disable ROM for TED
                sta TEDREG3F
                ;change memory config to: RAM,RAM,IO,RAM
                lda #MEM_CONFIG_RAM_IO_RAM
                sta R6510
                ;patch the kernal jump table to,
                ;prevent the ROM reactivation
                lda #$3F
                sta FF3EL
                ;disable kernal reset
                lda #<FakeNMI
                sta KERNALRESETL
                lda #>FakeNMI
                sta KERNALRESETH
                ;setup IRQ interrupt handler
                lda #<InterruptHandler
                sta CPUINTL
                lda #>InterruptHandler
                sta CPUINTH
                ;set the TED up for a raster IRQ interrupt
                lda #$00
                sta TEDREG0A
                ;setup raster line
                lda #(>V_SCAN_LINE)|$02
                sta TEDREG0A
                lda #<V_SCAN_LINE
                sta TEDREG0B
                ;hardware init
                jsr VideoRegSetup
                jsr AudioRegSetup
                ;clears all parameters at zero page
                jsr InitZeropage
                ;init the software timers
                jsr ResetAllTimers
                ;clear pending IRQs
                asl TEDREG09
                ;restart the interrupts
                cli
                jsr ClearScreen

;********************************************************************
; MAIN LOOP                                                         *
;********************************************************************
.main_loop
                ;Update all game objects.
                ;In order to keep everything in sync,
                ;sw timer counters, background startfield, bullets queue,
                ;sounds and tunes are updated during raster interrupt!
                jsr WaitForIrq
                ;lda #GRN
                ;sta TEDREG19
                jsr InputUpdate
                jsr AliensUpdate
                jsr MothershipUpdate
                jsr BirdsUpdate
                jsr PlayerUpdate
                jsr ShieldUpdate
                jsr ExplosionUpdate
                jsr GameFlowUpdate
                ;lda #BLK
                ;sta TEDREG19
                jmp .main_loop

;********************************************************************
; INCLUDE CODE/DATA MODULES                                         *
;********************************************************************
!source         "InterruptModule.asm"
!source         "TimerModule.asm"
!source         "ScreenModule.asm"
!source         "GeneralRoutines.asm"
!source         "StarsModule.asm"
!source         "InputModule.asm"
!source         "AliensModule.asm"
!source         "PlayerModule.asm"
!source         "ShieldModule.asm"
!source         "BulletsModule.asm"
!source         "FlowModule.asm"
!source         "AlienExplosionModule.asm"

* = $4000
!source         "CharacterSetA.asm"

* = $5000
!source         "CharacterSetB.asm"

* = $6000
!source         "CharacterSetC.asm"

!source         "ParticleExplosionModule.asm"
!source         "AlienBehaviorModule.asm"
!source         "BirdModule.asm"
!source         "MothershipModule.asm"
!source         "SoundModule.asm"

;********************************************************************
; MAIN MODULE END                                                   *
;********************************************************************